@@ -7,12 +7,12 @@ use bencher::Bencher;
 // cargo bench --bench lay_mine
 
 fn bench_laymine(bencher: &mut Bencher) {
-    bencher.iter(|| laymine(16, 30, 99, 8, 15, 0, 381, 2, 0));
-} // 23,721 ns/iter
+    bencher.iter(|| laymine(16, 30, 99, 8, 15));
+} // 10,951 ns/iter
 
 fn bench_laymine_op(bencher: &mut Bencher) {
-    bencher.iter(|| laymine_op(16, 30, 99, 8, 15, 0, 381, 100, 0));
-} // 24,000 ns/iter
+    bencher.iter(|| laymine_op(16, 30, 99, 8, 15));
+} // 11,522 ns/iter
 
 benchmark_group!(lay_mine, bench_laymine, bench_laymine_op);
 benchmark_main!(lay_mine);
