@@ -1,8 +1,8 @@
 use crate::utils::{
     cal3BV, cal3BV_exp, cal_table_minenum_enum, cal_table_minenum_recursion, chunk_matrixes,
-    combine, enuOneStep, enum_comb, laymine, laymine_op, legalize_board, refresh_board,
-    refresh_matrix, refresh_matrixs, refresh_matrixses, unsolvable_structure, BigNumber, C_query,
-    C,
+    combine, enuOneStep, enum_comb, find_a_border_cell, laymine, laymine_op, legalize_board,
+    refresh_board, refresh_matrix, refresh_matrixs, refresh_matrixses, unsolvable_structure,
+    BigNumber, C_query, C,
 };
 
 #[cfg(feature = "js")]
@@ -18,8 +18,12 @@ use rand::seq::SliceRandom;
 #[cfg(any(feature = "py", feature = "rs"))]
 use rand::thread_rng;
 
-// use rand::prelude::*;
-// use rand::{rngs::StdRng, Rng, SeedableRng};
+#[cfg(any(feature = "py", feature = "rs"))]
+use rand::prelude::*;
+#[cfg(any(feature = "py", feature = "rs"))]
+use rand::{rngs::StdRng, Rng, SeedableRng};
+
+use std::time;
 
 use std::cmp::{max, min};
 use std::sync::mpsc;
@@ -555,7 +559,8 @@ pub fn solve_enumerate(
                 for kk in &comb_relp_s[i][jj] {
                     not_mine.push(xs[i][*kk]);
                 }
-            } else if s_num == table_mine_num_i[1].iter().sum::<usize>() * comb_relp_s[i][jj].len() {
+            } else if s_num == table_mine_num_i[1].iter().sum::<usize>() * comb_relp_s[i][jj].len()
+            {
                 for kk in &comb_relp_s[i][jj] {
                     is_mine.push(xs[i][*kk]);
                 }
@@ -568,12 +573,12 @@ pub fn solve_enumerate(
 // 判断当前是否获胜
 // 游戏局面中必须没有标错的雷
 // 这个函数不具备普遍意义
-fn isVictory(BoardofGame: &Vec<Vec<i32>>, Board: &Vec<Vec<i32>>) -> bool {
-    let row = BoardofGame.len();
-    let col = BoardofGame[0].len();
+fn isVictory(board_of_game: &Vec<Vec<i32>>, Board: &Vec<Vec<i32>>) -> bool {
+    let row = board_of_game.len();
+    let col = board_of_game[0].len();
     for i in 0..row {
         for j in 0..col {
-            if BoardofGame[i][j] == 10 && Board[i][j] != -1 {
+            if board_of_game[i][j] == 10 && Board[i][j] != -1 {
                 return false;
             }
         }
@@ -589,42 +594,56 @@ pub fn is_solvable(Board: &Vec<Vec<i32>>, x0: usize, y0: usize) -> bool {
         //若包含不可判雷结构，则不是无猜
         return false;
     }
-    // println!("Board: {:?}", Board);
     let row = Board.len();
     let column = Board[0].len();
-    let mut BoardofGame = vec![vec![10; column]; row];
+    let mut board_of_game = vec![vec![10; column]; row];
     // 10是未打开，11是标雷
     // 局面大小必须超过6*6
-    refresh_board(&Board, &mut BoardofGame, vec![(x0, y0)]);
-    if isVictory(&BoardofGame, &Board) {
+    refresh_board(&Board, &mut board_of_game, vec![(x0, y0)]);
+    if isVictory(&board_of_game, &Board) {
         return true; // 暂且认为点一下就扫开也是可以的
     }
-    let mut not_mine;
     loop {
-        let (mut As, mut xs, mut bs, _, _) = refresh_matrixs(&BoardofGame);
-        let ans = solve_direct(&mut As, &mut xs, &mut bs, &mut BoardofGame);
-        not_mine = ans.0;
-        if !(not_mine.is_empty() && ans.1.is_empty()) {
-            let ans = solve_minus(&mut As, &mut xs, &mut bs, &mut BoardofGame);
-            not_mine = ans.0;
-            if !(not_mine.is_empty() && ans.1.is_empty()) {
+        let (mut As, mut xs, mut bs, _, _) = refresh_matrixs(&board_of_game);
+        let ans = solve_direct(&mut As, &mut xs, &mut bs, &mut board_of_game);
+        let mut not_mine;
+        if ans.0.is_empty() && ans.1.is_empty() {
+            let ans = solve_minus(&mut As, &mut xs, &mut bs, &mut board_of_game);
+            if ans.0.is_empty() && ans.1.is_empty() {
                 let ans = solve_enumerate(&As, &xs, &bs);
-                if !(ans.0.is_empty() && ans.1.is_empty()) {
+                if ans.0.is_empty() && ans.1.is_empty() {
                     return false;
+                } else {
+                    not_mine = ans.0
                 }
+                if !ans.1.is_empty() {
+                    for (o, p) in ans.1 {
+                        board_of_game[o][p] = 11;
+                    }
+                }
+            } else {
+                not_mine = ans.0
             }
+        } else {
+            not_mine = ans.0
         }
-        refresh_board(&Board, &mut BoardofGame, not_mine);
-        if isVictory(&BoardofGame, &Board) {
+        refresh_board(&Board, &mut board_of_game, not_mine);
+        if isVictory(&board_of_game, &Board) {
             return true;
         }
     }
 }
 
-/// 删选法多（8）线程无猜埋雷。  
-/// - 输入：3BV下限、上限，最大尝试次数。  
+/// 删选法多（8）线程无猜埋雷。对于雷密度很高的局面，多线程比单线程更快。  
+/// - 输入：高、宽、雷数、第几行、第几列、最大尝试次数。  
 /// - 返回: (是否成功、3BV、该线程的尝试次数)。  
 /// - 注意：若不成功返回最后生成的局面，此时则不一定无猜。
+/// - 局限：雷的密度无法设得很大。以高级的局面为例，雷数最多设到大约130左右。
+/// - 用python调用时的示例：
+/// ```python
+/// import ms_toollib as ms
+/// board = laymine_solvable_thread(16, 30, 99, 3, 20, 100000) # 在第3行、第20列开局
+/// ```
 #[cfg(any(feature = "py", feature = "rs"))]
 pub fn laymine_solvable_thread(
     row: usize,
@@ -688,9 +707,14 @@ pub fn laymine_solvable_thread(
 }
 
 /// 删选法单线程无猜埋雷。不可以生成任意雷密度的无猜局面。但雷满足均匀分布。  
-/// - 输入：高、宽、雷数、起手行数、起手列数、3BV下限、上限、最大尝试次数、最大枚举长度。  
+/// - 输入：高、宽、雷数、起手行数、起手列数、最大尝试次数。  
 /// - 返回：是否成功。  
 /// - 注意：若不成功返回最后生成的局面，此时则不一定无猜。
+/// - 用python调用时的示例：
+/// ```python
+/// import ms_toollib as ms
+/// board = laymine_solvable(16, 30, 99, 3, 20, 100000) # 在第3行、第20列开局
+/// ```
 pub fn laymine_solvable(
     row: usize,
     column: usize,
@@ -712,8 +736,9 @@ pub fn laymine_solvable(
     (Board, false)
 }
 
-/// 调整法无猜埋雷。可以生成任意雷密度的无猜局面。但雷不满足均匀分布。  
+/// 调整法无猜埋雷。可以生成任意雷密度的无猜局面。但雷不满足均匀分布。（还没写好）  
 /// - 返回局面、是否成功  
+// 局面中的-10代表还没埋雷。
 pub fn laymine_solvable_adjust(
     row: usize,
     column: usize,
@@ -740,153 +765,242 @@ pub fn laymine_solvable_adjust(
         let t = laymine(row, column, mine_num, x0, y0);
         return (t, false);
     }
-    'o: for time in 0..101 {
-        println!("尝试: {:?}", time);
-        board = vec![vec![0; column]; row];
-        let mut board_of_game = vec![vec![10; column]; row];
-        board_of_game[x0][y0] = 0;
-        if time == 100 {
-            return (laymine_op(row, column, mine_num, x0, y0), false);
-        }
-        let mut remain_mine_num = mine_num;
-        let mut remain_not_mine_num = row * column - area_op - mine_num;
-        let mut cells_plan_to_click = vec![];
-        for j in max(1, x0) - 1..min(row, x0 + 2) {
-            for k in max(1, y0) - 1..min(column, y0 + 2) {
-                if j != x0 || k != y0 {
-                    cells_plan_to_click.push((j, k));
-                }
+
+    board = vec![vec![-10; column]; row];
+    let mut board_of_game = vec![vec![10; column]; row];
+    board_of_game[x0][y0] = 0;
+    let remain_mine_num = mine_num;
+    let remain_not_mine_num = row * column - area_op - mine_num;
+    let mut cells_plan_to_click = vec![];
+    // 初始化第一步计划点开的格子
+    for j in max(1, x0) - 1..min(row, x0 + 2) {
+        for k in max(1, y0) - 1..min(column, y0 + 2) {
+            board[j][k] = 0;
+            if j != x0 || k != y0 {
+                cells_plan_to_click.push((j, k));
             }
         }
-
-        // let mut area_current_adjust = get_adjust_area(&board);
-        loop {
-            // 每一次代表点开一批格子
-            for &(x, y) in &cells_plan_to_click {
-                board_of_game[x][y] = 1;
-            }
-            cells_plan_to_click.clear();
-            let (Ases, xses, mut bses) = refresh_matrixses(&board_of_game);
-            if xses.is_empty() && remain_not_mine_num > 0 {
-                continue 'o;
-            }
-            for id in 0..xses.len() {
-                let xs_cell_num = xses[id].iter().fold(0, |acc, x| acc + x.len());
-                let mine_num_except = (xs_cell_num as f64 * remain_mine_num as f64
-                    / (remain_not_mine_num + remain_mine_num) as f64)
-                    as usize;
-                let mut success_flag = false;
-                'inner: for i in 0..xs_cell_num + 1 {
-                    let mine_num;
-                    if mine_num_except == 0 || xs_cell_num == 0 {
-                        mine_num = 0;
-                    } else if mine_num_except == xs_cell_num {
-                        mine_num = mine_num_except;
-                    } else if i == xs_cell_num - 1 {
-                        mine_num = 0;
-                    } else if i == xs_cell_num {
-                        mine_num = xs_cell_num;
-                    } else if xs_cell_num > mine_num_except * 2 {
-                        let z = mine_num_except * 2 - 1;
-                        if i < z {
-                            if i % 2 == 1 {
-                                mine_num = mine_num_except + (i + 1) / 2;
-                            } else {
-                                mine_num = mine_num_except - i / 2;
-                            }
-                        } else {
-                            mine_num = i + 1;
-                        }
-                    } else {
-                        let z = (xs_cell_num - mine_num_except) * 2 - 1;
-                        if i < z {
-                            if i % 2 == 1 {
-                                mine_num = mine_num_except + (i + 1) / 2;
-                            } else {
-                                mine_num = mine_num_except - i / 2;
-                            }
-                        } else {
-                            mine_num = xs_cell_num - i - 1;
-                        }
-                    }
-
-                    // let mine_num = (i + mine_num_except + 1) % (xs_cell_num + 1);
-                    if mine_num > remain_mine_num || xs_cell_num - mine_num > remain_not_mine_num {
-                        continue;
-                    }
-                    for _ in 0..5 {
-                        adjust_the_area_on_board(&mut board, &xses[id], mine_num);
-                        // 以下的循环用来修正b向量
-                        for bb in 0..bses[id].len() {
-                            for ss in 0..bses[id][bb].len() {
-                                bses[id][bb][ss] = 0;
-                                for aa in 0..Ases[id][bb][0].len() {
-                                    if Ases[id][bb][ss][aa] == 1 {
-                                        if board[xses[id][bb][aa].0][xses[id][bb][aa].1] == -1 {
-                                            bses[id][bb][ss] += 1;
-                                        }
-                                    }
-                                }
-                            }
-                        }
-                        let (mut n, i) = get_all_not_and_is_mine_on_board(
-                            &mut Ases[id].clone(),
-                            &mut xses[id].clone(),
-                            &mut bses[id].clone(),
-                            &mut board_of_game,
-                        );
-                        if n.len() > 0 || i.len() == xs_cell_num {
-                            success_flag = true;
-                            cells_plan_to_click.append(&mut n);
-                            remain_mine_num -= mine_num;
-                            remain_not_mine_num -= xs_cell_num - mine_num;
-                            for segment in &xses[id] {
-                                for &(xx, yy) in segment {
-                                    if board[xx][yy] == -1 {
-                                        board_of_game[xx][yy] = 11;
-                                    } else {
-                                        board_of_game[xx][yy] = 12;
-                                    }
-                                }
-                            }
-                            break 'inner;
-                        }
-                    }
-                }
-                if !success_flag {
-                    // 如果失败，代表没有合适的雷数，使得局面接下来无猜。
-                    continue 'o;
-                }
-            }
-            if remain_not_mine_num <= 0 {
-                // 进来代表埋雷成功。
-                // 按理不可能小于0，然而不自信。
-                for i in 0..row {
-                    for j in 0..column {
-                        if board_of_game[i][j] == 10 {
-                            board[i][j] = -1;
-                        }
-                    }
-                }
-                break 'o;
+    }
+    // 开始递归求解
+    let (mut b, flag) = adjust_step(
+        &board,
+        &board_of_game,
+        &cells_plan_to_click,
+        remain_mine_num,
+        remain_not_mine_num,
+    );
+    if !flag {
+        return (b, false);
+    }
+    for i in 0..row {
+        for j in 0..column {
+            if board[i][j] == -10 {
+                board[i][j] = -1;
             }
         }
     }
     // 最后，算数字
     for i in 0..row {
         for j in 0..column {
-            if board[i][j] == -1 {
+            if b[i][j] == -1 {
                 for m in max(1, i) - 1..min(row, i + 2) {
                     for n in max(1, j) - 1..min(column, j + 2) {
-                        if board[m][n] >= 0 {
-                            board[m][n] += 1;
+                        if b[m][n] >= 0 {
+                            b[m][n] += 1;
                         }
                     }
                 }
             }
         }
     }
-    (board, true)
+    (b, flag)
+}
+
+// 调整法的递归部分。注意空间复杂度为局面面积乘求解步数。
+// 返回没有计算数字的局面和是否成功。
+fn adjust_step(
+    board: &Vec<Vec<i32>>,            // 当前的board，数字没有计算，只有0，-1，-10
+    board_of_game: &Vec<Vec<i32>>,    // 当前的board_of_game，数字没有计算，只有10，1
+    plan_click: &Vec<(usize, usize)>, // 当前计划点开的格子，递归部分要保证点开后，局面是有解开的可能的
+    remain_mine_num: usize,           // 当前还要埋的雷数
+    remain_not_mine_num: usize,       // 当前还要埋的非雷数
+) -> (Vec<Vec<i32>>, bool) {
+    let mut b = board.clone(); // 克隆一个board的备份
+    let mut bg = board_of_game.clone(); // 克隆一个board_of_game的备份
+    let mut r = remain_mine_num; // 克隆一个当前还要埋的雷数的备份
+    let mut rn = remain_not_mine_num; // 克隆一个当前还要埋的非雷数的备份
+    for (x, y) in plan_click {
+        bg[*x][*y] = 1;
+    } // 点开当前的board_of_game的备份上的计划点开的格子，用1临时表示
+    println!(">>>>>>");
+    bg.iter().for_each(|i| {
+        i.iter()
+            .for_each(|j| print!("{number:>width$}", number = j, width = 4));
+        println!("")
+    });
+    println!("\\");
+    b.iter().for_each(|i| {
+        i.iter()
+            .for_each(|j| print!("{number:>width$}", number = j, width = 4));
+        println!("")
+    });
+    std::io::stdin().read_line(&mut "".to_string());
+    // thread::sleep(time::Duration::from_millis(2000));
+    let mut plan_click = vec![];
+    let (Ases, xses, mut bses) = refresh_matrixses(&bg);
+    // 所有分支的前沿都已遍历完成。
+
+    // println!("Ases: {:?}", Ases);
+    // println!("xses: {:?}", xses);
+    // println!("bses: {:?}", bses);
+    // println!("rn: {:?}", rn);
+    // 前沿全部被雷堵塞，无法继续
+    if xses.is_empty() {
+        if rn > 0 {
+            return (vec![], false);
+        } else {
+            return (b, true);
+        }
+    }
+    // 只对第一块处理。缺点是内存复杂度上升
+    let As_0 = Ases.get(0).unwrap();
+    let xs_0 = xses.get(0).unwrap();
+    let bs_0 = bses.get_mut(0).unwrap();
+
+    // let mut front_xs_0 = xs_0.clone();
+    let mut front_xs_0 = xs_0.clone();
+    front_xs_0
+        .iter_mut()
+        .for_each(|p| p.retain(|x| b[x.0][x.1] == -10));
+    // 前沿格子————此格子在边缘，且是没有埋过雷的
+    let xs_cell_num = front_xs_0.iter().fold(0, |acc, x| acc + x.len());
+    let mine_num_except = (xs_cell_num as f64 * r as f64 / (rn + r) as f64) as usize;
+    let mut success_flag = false;
+    // 对不同雷数循环
+    'inner: for i in 0..xs_cell_num + 1 {
+        // 根据算法的映射，计算出mine_num，计划的埋雷量
+        let mine_num;
+        if mine_num_except == 0 || xs_cell_num == 0 {
+            mine_num = 0;
+        } else if mine_num_except == xs_cell_num {
+            mine_num = mine_num_except;
+        } else if i == xs_cell_num - 1 {
+            mine_num = 0;
+        } else if i == xs_cell_num {
+            mine_num = xs_cell_num;
+        } else if xs_cell_num > mine_num_except * 2 {
+            let z = mine_num_except * 2 - 1;
+            if i < z {
+                if i % 2 == 1 {
+                    mine_num = mine_num_except + (i + 1) / 2;
+                } else {
+                    mine_num = mine_num_except - i / 2;
+                }
+            } else {
+                mine_num = i + 1;
+            }
+        } else {
+            let z = (xs_cell_num - mine_num_except) * 2 - 1;
+            if i < z {
+                if i % 2 == 1 {
+                    mine_num = mine_num_except + (i + 1) / 2;
+                } else {
+                    mine_num = mine_num_except - i / 2;
+                }
+            } else {
+                mine_num = xs_cell_num - i - 1;
+            }
+        }
+        // let mine_num = (i + mine_num_except + 1) % (xs_cell_num + 1);
+        // 排除显而易见不可能的情况。
+        if mine_num > r || xs_cell_num - mine_num > rn {
+            continue;
+        }
+        // 对每种雷数，重复尝试3次。
+        for _ in 0..3 {
+            adjust_the_area_on_board(&mut b, &front_xs_0, mine_num);
+            // 以下的循环用来修正b向量
+            for bb in 0..bs_0.len() {
+                for ss in 0..bs_0[bb].len() {
+                    // ss是第几个方程
+                    bs_0[bb][ss] = 0;
+                    for aa in 0..As_0[bb][0].len() {
+                        // aa是第几个格子
+                        if As_0[bb][ss][aa] == 1 {
+                            if b[xs_0[bb][aa].0][xs_0[bb][aa].1] == -1
+                                && bg[xs_0[bb][aa].0][xs_0[bb][aa].1] != 11
+                            {
+                                bs_0[bb][ss] += 1;
+                            }
+                        }
+                    }
+                }
+            }
+            let (mut n, i) = get_all_not_and_is_mine_on_board(
+                &mut As_0.clone(),
+                &mut xs_0.clone(),
+                &mut bs_0.clone(),
+                &mut bg,
+            );
+            n.iter().for_each(|x| bg[x.0][x.1] = 1);
+            // i.iter().for_each(|x| bg[x.0][x.1] = 10);
+            println!("非雷: {:?}", n);
+            println!("A矩阵: {:?}", As_0);
+            println!("x矩阵: {:?}", xs_0);
+            println!("b矩阵: {:?}", bs_0);
+            if n.len() > 0 || i.len() == xs_cell_num {
+                // n.iter().for_each(|x| bg[x.0][x.1] = 12);
+                i.iter().for_each(|x| bg[x.0][x.1] = 11);
+                success_flag = true; // 当前步骤成功
+                plan_click.append(&mut n);
+                // println!("xs_cell_num: {:?}", xs_cell_num);
+                // println!("mine_num: {:?}", mine_num);
+                // println!("rn: {:?}", rn);
+                // println!("r: {:?}", r);
+                r -= mine_num;
+                rn -= xs_cell_num - mine_num;
+                // for segment in xs_0 {
+                //     // xxx
+                //     for &(xx, yy) in segment {
+                //         if b[xx][yy] == -1 {
+                //             bg[xx][yy] = 11;
+                //         } else {
+                //             bg[xx][yy] = 12;
+                //         }
+                //     }
+                // }
+                println!("555");
+                let a = adjust_step(&b, &bg, &plan_click, r, rn);
+                println!("666");
+                if a.1 {
+                    println!("777");
+                    b = a.0;
+                    break 'inner;
+                } else {
+                    println!("888");
+                    b = board.clone();
+                    bg = board_of_game.clone();
+                    continue 'inner;
+                }
+            }
+        }
+    }
+    if !success_flag {
+        // 如果失败，代表没有合适的雷数，使得局面接下来无猜。
+        return (vec![], false);
+    }
+    if rn <= 0 {
+        return (b, true);
+    } else {
+        return (vec![], false);
+        // return adjust_step(
+        //     &b,
+        //     &bg,
+        //     &plan_click,
+        //     r,
+        //     rn,
+        // );
+    }
 }
 
 // 在指定的局部（area_current_adjust）埋雷，不刷新board上的数字
@@ -902,8 +1016,8 @@ fn adjust_the_area_on_board(
     b.append(&mut vec![-1; mine_num]);
 
     #[cfg(any(feature = "py", feature = "rs"))]
-    let mut rng = thread_rng();
-    // let mut rng = StdRng::seed_from_u64(2);
+    // let mut rng = thread_rng();
+    let mut rng = StdRng::seed_from_u64(2);
     #[cfg(any(feature = "py", feature = "rs"))]
     b.shuffle(&mut rng);
 
@@ -919,7 +1033,7 @@ fn adjust_the_area_on_board(
     }
 }
 
-// 对高级3BV做采样。16线程。
+/// 埋雷并计算高级局面3BV的引擎，用于研究高级3BV的分布。16线程。传入局数，例如1000 000。试一下你的电脑算的有多块吧。  
 #[cfg(any(feature = "py", feature = "rs"))]
 pub fn sample_3BVs_exp(x0: usize, y0: usize, n: usize) -> [usize; 382] {
     // 从标准高级中采样计算3BV
@@ -941,7 +1055,6 @@ pub fn sample_3BVs_exp(x0: usize, y0: usize, n: usize) -> [usize; 382] {
 
 #[cfg(any(feature = "py", feature = "rs"))]
 fn laymine_study_exp(x0: usize, y0: usize, n: usize) -> [usize; 382] {
-    // 专用埋雷并计算3BV引擎，用于研究
     let mut rng = thread_rng();
     // let area: usize = 16 * 30 - 1;
     let pointer = x0 + y0 * 16;
@@ -1025,9 +1138,10 @@ fn OBR_cell(
 
 /// <span id="OBR_board">光学局面识别引擎。  
 /// - 输入：依次为列向量形式的三通道的像素数据、图像的高度、宽度。  
+/// - 性能：识别的成功率不是百分之百。识别失败时，甚至可能出现不可恢复的错误。想提高成功率，需要满足：图片清晰、格子的宽度在8到300像素之间、图片中没有鼠标遮挡。  
 /// - 以下提供一段用python调用时的示例：  
 /// ```python
-/// # pip install ms_toollib --upgrade
+/// # pip install ms_toollib==1.3.6（windows和linux不一样，请查看[主页](https://github.com/eee555/ms_toollib)）
 /// import ms_toollib
 /// import matplotlib.image as mpimg
 /// import numpy as np
@@ -1117,55 +1231,88 @@ pub fn get_all_not_and_is_mine_on_board(
     (not_mine, is_mine)
 }
 
-/// 判断是否为可能可以（区别于必然可以）判雷时的猜雷；对应弱无猜规则。  
+/// 判断是否为可能可以（区别于必然可以）判雷时的猜雷；对应弱无猜、准无猜规则。  
 /// - 前提：点在未知格上，即10。  
-/// 情况表：1 -> 正确的判雷。  
-/// 2 -> 必要的猜雷。  
+/// - 约定：1 -> 正确的判雷。  
+///  - 注意：不可以处理14、15等标记（全当成10）。输入为玩家维护的游戏局面，因此会首先清干净玩家标的雷。  
+/// 2 -> 必要的猜雷。（由于全局或局部不可判而猜雷）  
 /// 3 -> 不必要的猜雷。  
-/// 4 -> 踩到必然的雷。
+/// 4 -> 踩到必然的雷。  
+/// 5 -> 没有结果。因为此处已经被点开了。
 pub fn is_guess_while_needless(board_of_game: &mut Vec<Vec<i32>>, xy: &(usize, usize)) -> i32 {
+    board_of_game.iter_mut().for_each(|x| {
+        x.iter_mut().for_each(|xx| {
+            if *xx > 10 {
+                *xx = 10
+            }
+        })
+    });
+    if board_of_game[xy.0][xy.1] < 10 {
+        return 5;
+    }
     let mut flag_need = true;
     let (mut Ases, mut xses, mut bses) = refresh_matrixses(&board_of_game);
-    let t = xses
-        .iter()
-        .position(|r| r.iter().any(|x| x.contains(&xy)))
-        .unwrap();
-    let mut As = &mut Ases[t];
-    let mut xs = &mut xses[t];
-    let mut bs = &mut bses[t];
-    let (n, _) = solve_direct(As, xs, bs, board_of_game);
-    flag_need = n.is_empty();
-    match board_of_game[xy.0][xy.1] {
-        12 => return 1,
-        11 => return 4,
-        _ => {
-            let (n, _) = solve_minus(As, xs, bs, board_of_game);
-            flag_need = flag_need || n.is_empty();
-            match board_of_game[xy.0][xy.1] {
-                12 => return 1,
-                11 => return 4,
-                _ => {
-                    let (n, i) = solve_enumerate(As, xs, bs);
-                    flag_need = flag_need || n.is_empty();
-                    if n.contains(xy) {
-                        return 1;
-                    } else if i.contains(xy) {
-                        return 4;
-                    } else if flag_need {
-                        return 2;
-                    } else {
-                        return 3;
+    if let (Some(xy), flag_border) = find_a_border_cell(board_of_game, xy) {
+        let t = xses
+            .iter()
+            .position(|r| r.iter().any(|x| x.contains(&xy)))
+            .unwrap();
+        let mut As = &mut Ases[t];
+        let mut xs = &mut xses[t];
+        let mut bs = &mut bses[t];
+        let (n, _) = solve_direct(As, xs, bs, board_of_game);
+        if !flag_border && !n.is_empty() {
+            return 3;
+        }
+        flag_need = n.is_empty();
+        match board_of_game[xy.0][xy.1] {
+            12 => return 1,
+            11 => return 4,
+            _ => {
+                let (n, _) = solve_minus(As, xs, bs, board_of_game);
+                if !flag_border && !n.is_empty() {
+                    return 3;
+                }
+                flag_need = flag_need && n.is_empty();
+                match board_of_game[xy.0][xy.1] {
+                    12 => return 1,
+                    11 => return 4,
+                    _ => {
+                        let (n, i) = solve_enumerate(As, xs, bs);
+                        if !flag_border && !n.is_empty() {
+                            return 3;
+                        }
+                        flag_need = flag_need && n.is_empty();
+                        if n.contains(&xy) {
+                            return 1;
+                        } else if i.contains(&xy) {
+                            return 4;
+                        } else if flag_need {
+                            return 2;
+                        } else {
+                            return 3;
+                        }
                     }
                 }
             }
         }
+    } else {
+        return 2; // 无论何时，包心雷，是合理的猜雷。
     }
 }
 
 /// 判断是否为判雷；对应强无猜规则。
 /// - 前提：对打开非雷或标记是雷的行为判断。   
-/// - 不仅可以判断是雷，也可以判断非雷。
+/// - 不仅可以判断是雷，也可以判断非雷。  
+/// - 注意：不可以处理14、15等标记（当成10处理）。输入为玩家维护的游戏局面，因此会首先清干净玩家标的雷。  
 pub fn is_able_to_solve(board_of_game: &mut Vec<Vec<i32>>, xy: &(usize, usize)) -> bool {
+    board_of_game.iter_mut().for_each(|x| {
+        x.iter_mut().for_each(|xx| {
+            if *xx > 10 {
+                *xx = 10
+            }
+        })
+    });
     let (mut As, mut xs, mut bs, _, _) = refresh_matrixs(&board_of_game);
     solve_direct(&mut As, &mut xs, &mut bs, board_of_game);
     if board_of_game[xy.0][xy.1] == 11 || board_of_game[xy.0][xy.1] == 12 {
