@@ -3,15 +3,16 @@
 };
 use crate::analyse_methods::{
     analyse_high_risk_guess, analyse_jump_judge, analyse_mouse_trace, analyse_needless_guess,
-    analyse_vision_transfer, analyse_survive_poss
+    analyse_survive_poss, analyse_vision_transfer,
 };
 use crate::utils::{refresh_board, refresh_matrixs};
 use std::cmp::{max, min};
 use std::fs;
 
-/// 局面状态机，分析操作与局面的交互、推衍局面。在线地统计左右双击次数、ce次数、左键、右键、双击、当前解决的3BV。
-/// - 局限：目前不能计算path。  
+/// 局面状态机，侧重分析操作与局面的交互、推衍局面。在线地统计左右双击次数、ce次数、左键、右键、双击、当前解决的3BV。  
+/// - 局限：不关注具体的线路，因此不能计算path等。  
 /// - 注意：ce的计算与扫雷网是不同的，本工具箱中，重复标同一个雷只算一个ce，即反复标雷、取消标雷不算作ce。
+/// 应用场景：强化学习训练AI、游戏复盘计算指标。不能处理高亮（18）、算法确定是雷（12）等标记。  
 /// - 用python调用时的示例：
 /// ```python
 /// import ms_toollib as ms
@@ -36,7 +37,7 @@ use std::fs;
 /// print('局面: ', v.game_board)
 /// ```
 pub struct MinesweeperBoard {
-    board: Vec<Vec<i32>>,
+    pub board: Vec<Vec<i32>>,
     /// 局面
     pub game_board: Vec<Vec<i32>>,
     flagedList: Vec<(usize, usize)>, // 记录哪些雷曾经被标过，则再标这些雷不记为ce
@@ -52,9 +53,13 @@ pub struct MinesweeperBoard {
     pub flag: usize,
     /// 已解决的3BV数
     pub solved3BV: usize,
-    row: usize,
-    column: usize,
-    mouse_state: MouseState,
+    pub row: usize,
+    pub column: usize,
+    pub mouse_state: MouseState,
+    pub game_board_state: GameBoardState,
+    // 指针，用于判断局面是否全部扫开
+    pointer_x: usize,
+    pointer_y: usize,
 }
 
 impl MinesweeperBoard {
@@ -74,6 +79,9 @@ impl MinesweeperBoard {
             solved3BV: 0,
             flagedList: vec![],
             mouse_state: MouseState::UpUp,
+            game_board_state: GameBoardState::Ready,
+            pointer_x: 0,
+            pointer_y: 0,
         }
     }
     fn left_click(&mut self, x: usize, y: usize) -> Result<u8, ()> {
@@ -81,20 +89,31 @@ impl MinesweeperBoard {
         if self.game_board[x][y] != 10 {
             return Ok(0);
         }
+        refresh_board(&self.board, &mut self.game_board, vec![(x, y)]);
         match self.board[x][y] {
             0 => {
                 self.solved3BV += 1;
                 self.ces += 1;
-                refresh_board(&self.board, &mut self.game_board, vec![(x, y)]);
+                // refresh_board(&self.board, &mut self.game_board, vec![(x, y)]);
+                if self.is_win() {
+                    self.game_board_state = GameBoardState::Win;
+                }
                 Ok(2)
             }
-            -1 => Err(()),
+            -1 => {
+                // refresh_board(&self.board, &mut self.game_board, vec![(x, y)]);
+                self.game_board_state = GameBoardState::Loss;
+                Ok(0)
+            }
             _ => {
-                refresh_board(&self.board, &mut self.game_board, vec![(x, y)]);
+                // refresh_board(&self.board, &mut self.game_board, vec![(x, y)]);
                 if self.num_is_3BV(x, y) {
                     self.solved3BV += 1;
                 }
                 self.ces += 1;
+                if self.is_win() {
+                    self.game_board_state = GameBoardState::Win;
+                }
                 Ok(2)
             }
         }
@@ -152,14 +171,15 @@ impl MinesweeperBoard {
                     if self.game_board[i][j] == 11 {
                         flagedNum += 1
                     }
-                    if self.game_board[i][j] == 10 && self.board[i][j] != -1 {
-                        if self.board[i][j] == 0 {
-                            flag_ch_op = true;
-                        }
-                        flagChordingUseful = true;
+                    if self.game_board[i][j] == 10 {
                         chordingCells.push((i, j));
-                        if self.num_is_3BV(i, j) {
-                            surround3BV += 1;
+                        flagChordingUseful = true;
+                        if self.board[i][j] > 0 {
+                            if self.num_is_3BV(i, j) {
+                                surround3BV += 1;
+                            }
+                        } else if self.board[i][j] == 0 {
+                            flag_ch_op = true;
                         }
                     }
                 }
@@ -171,15 +191,24 @@ impl MinesweeperBoard {
             if flag_ch_op {
                 self.solved3BV += 1;
             }
+            for ch in &chordingCells {
+                if self.board[ch.0][ch.1] == -1 {
+                    self.game_board_state = GameBoardState::Loss;
+                }
+            }
             refresh_board(&self.board, &mut self.game_board, chordingCells);
+            if self.is_win() {
+                self.game_board_state = GameBoardState::Win;
+            }
             Ok(2)
         } else {
             Ok(0)
         }
     }
     fn num_is_3BV(&self, x: usize, y: usize) -> bool {
-        // 判断该数字是不是3BV，0也可以
-        if self.board[x][y] == -1 {
+        // 判断该大于0的数字是不是3BV
+        // 如果是0，即使是3bv，依然返回false
+        if self.board[x][y] <= 0 {
             return false;
         }
         for i in max(1, x) - 1..min(self.row, x + 2) {
@@ -191,37 +220,85 @@ impl MinesweeperBoard {
         }
         true
     }
+    /// 返回的值的含义是：0：没有任何作用的操作，例如左键数字、踩雷。  
+    /// 1：推进了局面，但没有改变ai对局面的判断，特指标雷。  
+    /// 2：改变局面的操作，左键、双击。  
+    /// e的类型有6种，lc, lr, rc, rr, cc, cr；这和arbiter等软件是不同的，本工具箱兼容了他们  
+    /// cc对应的是，按下第二个键，至于按的哪个不知道  
+    /// crl对应的是，两个键都按下时，抬起左键(chording_release_left)，和lr等价，可用可不用  
+    /// crr对应的是，两个键都按下时，抬起右键(chording_release_right)，和rr等价，可用可不用  
+    /// 在严格的鼠标状态机中，有些情况是不可能的，例如右键没有抬起就按下两次，但在阿比特中就观察到这种事情。
+    /// 因此此类情况不再简单地报出不可恢复地错误，而是若无其事地继续解析。  
     pub fn step(&mut self, e: &str, pos: (usize, usize)) -> Result<u8, ()> {
-        if pos.0 == self.row && pos.1 == self.column {
-            self.mouse_state = MouseState::UpUp;
-            return Ok(0u8)
+        match self.game_board_state {
+            GameBoardState::Ready => match e {
+                "lr" => match self.mouse_state {
+                    // 鼠标状态与局面状态的耦合
+                    MouseState::DownUp => self.game_board_state = GameBoardState::Playing,
+                    _ => {}
+                },
+                _ => {}
+            },
+            GameBoardState::Playing => {}
+            _ => return Ok(0),
         }
+        // if pos.0 == self.row && pos.1 == self.column {
+        //     // 发生这种事情，是由于阿比特会把点到局面外，改成点到最右下角
+        //     self.mouse_state = MouseState::UpUp;
+        //     return Ok(0);
+        // }
+        // if pos.0 >= self.row || pos.1 >= self.column {
+        //     // 越界错误，未定义的行为，不可恢复的错误
+        //     return Err(());
+        // }
         match e {
             "lc" => match self.mouse_state {
                 MouseState::UpUp => self.mouse_state = MouseState::DownUp,
                 MouseState::UpDown => self.mouse_state = MouseState::Chording,
                 MouseState::UpDownNotFlag => self.mouse_state = MouseState::ChordingNotFlag,
-                _ => return Err(()),
+                // 以下情况其实是不可能的
+                MouseState::DownUp => {}
+                MouseState::DownUpAfterChording => {}
+                MouseState::Chording => {}
+                MouseState::ChordingNotFlag => {}
+                MouseState::Undefined => self.mouse_state = MouseState::DownUp,
             },
             "lr" => match self.mouse_state {
                 MouseState::DownUp => {
                     self.mouse_state = MouseState::UpUp;
+                    if pos.0 == self.row && pos.1 == self.column {
+                        return Ok(0);
+                    }
                     return self.left_click(pos.0, pos.1);
                 }
                 MouseState::Chording => {
                     self.mouse_state = MouseState::UpDown;
+                    if pos.0 == self.row && pos.1 == self.column {
+                        return Ok(0);
+                    }
                     return self.chording_click(pos.0, pos.1);
                 }
                 MouseState::DownUpAfterChording => self.mouse_state = MouseState::UpUp,
                 MouseState::ChordingNotFlag => {
                     self.mouse_state = MouseState::UpDown;
                     self.right -= 1;
+                    if pos.0 == self.row && pos.1 == self.column {
+                        return Ok(0);
+                    }
                     return self.chording_click(pos.0, pos.1);
                 }
-                _ => return Err(()),
+                // 以下情况其实是不可能的
+                MouseState::UpDown => {}
+                MouseState::UpDownNotFlag => {}
+                MouseState::UpUp => self.mouse_state = MouseState::UpUp,
+                MouseState::Undefined => self.mouse_state = MouseState::UpUp,
             },
             "rc" => match self.mouse_state {
                 MouseState::UpUp => {
+                    if pos.0 == self.row && pos.1 == self.column {
+                        self.mouse_state = MouseState::UpDownNotFlag;
+                        return Ok(0);
+                    }
                     if self.game_board[pos.0][pos.1] < 10 {
                         self.mouse_state = MouseState::UpDownNotFlag;
                     } else {
@@ -231,18 +308,76 @@ impl MinesweeperBoard {
                 }
                 MouseState::DownUp => self.mouse_state = MouseState::Chording,
                 MouseState::DownUpAfterChording => self.mouse_state = MouseState::Chording,
-                _ => return Err(()),
+                // 以下情况其实是不可能的
+                MouseState::UpDown => {}
+                MouseState::UpDownNotFlag => {}
+                MouseState::Chording => {}
+                MouseState::ChordingNotFlag => {}
+                MouseState::Undefined => self.mouse_state = MouseState::UpDown,
             },
             "rr" => match self.mouse_state {
                 MouseState::UpDown => self.mouse_state = MouseState::UpUp,
                 MouseState::UpDownNotFlag => self.mouse_state = MouseState::UpUp,
                 MouseState::Chording => {
                     self.mouse_state = MouseState::DownUpAfterChording;
+                    if pos.0 == self.row && pos.1 == self.column {
+                        return Ok(0);
+                    }
                     return self.chording_click(pos.0, pos.1);
                 }
                 MouseState::ChordingNotFlag => {
                     self.mouse_state = MouseState::DownUpAfterChording;
                     self.right -= 1;
+                    if pos.0 == self.row && pos.1 == self.column {
+                        return Ok(0);
+                    }
+                    return self.chording_click(pos.0, pos.1);
+                }
+                // 以下情况其实是不可能的
+                MouseState::DownUp => {}
+                MouseState::DownUpAfterChording => {}
+                MouseState::UpUp => self.mouse_state = MouseState::UpUp,
+                MouseState::Undefined => self.mouse_state = MouseState::UpUp,
+            },
+            "cc" => match self.mouse_state {
+                MouseState::DownUp => self.mouse_state = MouseState::Chording,
+                MouseState::DownUpAfterChording => self.mouse_state = MouseState::Chording,
+                MouseState::UpDown => self.mouse_state = MouseState::Chording,
+                MouseState::UpDownNotFlag => self.mouse_state = MouseState::ChordingNotFlag,
+                _ => return Err(()),
+            },
+            "crl" => match self.mouse_state {
+                MouseState::Chording => {
+                    self.mouse_state = MouseState::UpDown;
+                    if pos.0 == self.row && pos.1 == self.column {
+                        return Ok(0);
+                    }
+                    return self.chording_click(pos.0, pos.1);
+                }
+                MouseState::ChordingNotFlag => {
+                    self.mouse_state = MouseState::UpDown;
+                    self.right -= 1;
+                    if pos.0 == self.row && pos.1 == self.column {
+                        return Ok(0);
+                    }
+                    return self.chording_click(pos.0, pos.1);
+                }
+                _ => return Err(()),
+            },
+            "crr" => match self.mouse_state {
+                MouseState::Chording => {
+                    self.mouse_state = MouseState::DownUpAfterChording;
+                    if pos.0 == self.row && pos.1 == self.column {
+                        return Ok(0);
+                    }
+                    return self.chording_click(pos.0, pos.1);
+                }
+                MouseState::ChordingNotFlag => {
+                    self.mouse_state = MouseState::DownUpAfterChording;
+                    self.right -= 1;
+                    if pos.0 == self.row && pos.1 == self.column {
+                        return Ok(0);
+                    }
                     return self.chording_click(pos.0, pos.1);
                 }
                 _ => return Err(()),
@@ -251,31 +386,76 @@ impl MinesweeperBoard {
         }
         Ok(0)
     }
+    /// 直接分析整局的操作流，中间也可以停顿
+    /// 开始游戏前的任何操作也都记录次数
     pub fn step_flow(&mut self, operation: Vec<(&str, (usize, usize))>) -> Result<(), ()> {
-        // 直接分析整局的操作流，中间也可以停顿
         for op in operation {
             self.step(op.0, op.1)?;
         }
         Ok(())
     }
-    // pub fn reset(&self) {
-    //     // 重载，暂时没用不写
-    // }
+    fn is_win(&mut self) -> bool {
+        for j in self.pointer_y..self.column {
+            if self.game_board[self.pointer_x][j] >= 10 && self.board[self.pointer_x][j] != -1 {
+                self.pointer_y = j;
+                return false;
+            }
+        }
+        for i in self.pointer_x + 1..self.row {
+            for j in 0..self.column {
+                if self.game_board[i][j] >= 10 && self.board[i][j] != -1 {
+                    self.pointer_x = i;
+                    self.pointer_y = j;
+                    return false;
+                }
+            }
+        }
+        true
+    }
+    /// 初始化。对应强化学习领域gym的api中的reset。
+    pub fn reset(&mut self) {
+        self.game_board = vec![vec![10; self.column]; self.row];
+        self.board = vec![vec![0; self.column]; self.row];
+        self.left = 0;
+        self.right = 0;
+        self.chording = 0;
+        self.ces = 0;
+        self.flag = 0;
+        self.left = 0;
+        self.solved3BV = 0;
+        self.flagedList = vec![];
+        self.mouse_state = MouseState::UpUp;
+        self.game_board_state = GameBoardState::Ready;
+        self.pointer_x = 0;
+        self.pointer_y = 0;
+    }
 }
 
-// #[derive(Debug, PartialEq)]
-// pub struct UpUp;
-
-#[derive(Debug, PartialEq)]
-enum MouseState {
-    // 鼠标状态机
+/// 鼠标状态机
+#[derive(Debug, PartialEq, Clone)]
+pub enum MouseState {
     UpUp,
     UpDown,
-    UpDownNotFlag, // 右键按下，且没有标雷的状态
+    /// 右键按下，且既没有标雷，也没有取消标雷的状态
+    UpDownNotFlag,
     DownUp,
-    Chording,            // 双键都按下，的其他状态
-    ChordingNotFlag,     // 双键都按下，且是在不可以右击的格子上、先按下右键
-    DownUpAfterChording, //双击后先弹起右键，左键还没弹起的状态
+    /// 双键都按下的其他状态
+    Chording,
+    /// 双键都按下，且是在不可以右击的格子上、先按下右键
+    ChordingNotFlag,
+    /// 双击后先弹起右键，左键还没弹起的状态
+    DownUpAfterChording,
+    /// 未初始化的状态
+    Undefined,
+}
+
+/// 游戏局面状态机
+#[derive(Debug, PartialEq)]
+pub enum GameBoardState {
+    Ready,
+    Playing,
+    Loss,
+    Win,
 }
 
 #[derive(Debug)]
@@ -293,16 +473,21 @@ pub enum ErrReadVideoReason {
 pub struct VideoEvent {
     pub time: f64,
     pub mouse: String,
-    pub x: u16, // 单位是像素不是格
+    /// 单位是像素不是格
+    pub x: u16,
     pub y: u16,
-    pub useful_level: u8, // 0代表完全没用，
-    // 1代表能仅推进局面但不改变对局面的后验判断，例如标雷和取消标雷
-    // 2代表改变对局面的后验判断的操作
-    // 和ce没有关系，仅用于控制计算
+    /// 0代表完全没用，
+    /// 1代表能仅推进局面但不改变对局面的后验判断，例如标雷和取消标雷
+    /// 2代表改变对局面的后验判断的操作
+    /// 和ce没有关系，仅用于控制计算
+    pub useful_level: u8,
+    /// 当前局面。鼠标move是不会存的。
     pub posteriori_game_board: GameBoard,
     // pub posteriori_game_board: Vec<Vec<i32>>,
     // pub posteriori_board_poss: Vec<Vec<f64>>,
     pub comments: String,
+    /// 该操作完成以后的鼠标状态。和录像高亮有关。即使是鼠标move也会记录。
+    pub mouse_state: MouseState,
 }
 
 pub struct StaticParams {
@@ -337,6 +522,9 @@ pub struct DynamicParams {
     pub chordings_s: f64,
     pub clicks_s: f64,
     pub ces_s: f64,
+    pub ioe: f64,
+    pub corr: f64,
+    pub thrp: f64,
 }
 
 trait BaseParser {
@@ -348,7 +536,7 @@ trait BaseParser {
 
 /// avf录像解析器。  
 /// - 功能：解析avf格式的录像，有详细分析录像的方法。  
-/// - 以下是在python中调用的示例。
+/// - 以下是在python中调用的示例。  
 /// ```python
 /// v = ms.AvfVideo("arbiter_beg.avf");
 /// v.parse_video()
@@ -356,7 +544,7 @@ trait BaseParser {
 /// print(v.bbbv)
 /// print(v.clicks)
 /// print(v.clicks_s)
-/// v.analyse_for_features(["high_risk_guess"])
+/// v.analyse_for_features(["high_risk_guess"]) # 用哪些分析方法。分析结果会记录到events.comments里
 /// for i in range(v.events_len):
 ///     print(v.events_time(i), v.events_x(i), v.events_y(i), v.events_mouse(i))
 /// for i in range(v.events_len):
@@ -364,6 +552,7 @@ trait BaseParser {
 ///         print(v.events_posteriori_game_board(i).poss)
 /// ```
 pub struct AvfVideo {
+    /// 文件名
     pub file_name: String,
     pub width: usize,
     pub height: usize,
@@ -372,10 +561,19 @@ pub struct AvfVideo {
     pub level: usize,
     pub board: Vec<Vec<i32>>,
     pub events: Vec<VideoEvent>,
+    /// 录像播放时的指针，播放哪一帧
+    pub current_event_id: usize,
+    /// 录像标识
     pub player: String,
+    /// 游戏起始时间和中止时间。不整理格式，读成字符串。
+    /// 举例：在阿比特中，‘16.10.2021.22.24.23.9906’，意味2021年10月16日，下午10点24分23秒9906。
+    pub start_time: String,
+    pub end_time: String,
     video_data: Vec<u8>,
     offset: usize,
+    /// 可以计算静态指标（有些没写完）
     pub static_params: StaticParams,
+    /// 可以动态静态指标（有些没写完）
     pub dynamic_params: DynamicParams,
 }
 
@@ -406,9 +604,10 @@ impl BaseParser for AvfVideo {
     }
 }
 impl AvfVideo {
+    #[cfg(any(feature = "py", feature = "rs"))]
     pub fn new(file_name: &str) -> AvfVideo {
         let video_data: Vec<u8> = fs::read(file_name).unwrap();
-        // for i in 42642 - 500..42641 {
+        // for i in 0..500 {
         //     print!("{:?}", video_data[i] as char);
         // }
         AvfVideo {
@@ -420,7 +619,10 @@ impl AvfVideo {
             level: 0,
             board: vec![],
             events: vec![],
+            current_event_id: 0,
             player: "".to_string(),
+            start_time: "".to_string(),
+            end_time: "".to_string(),
             video_data: video_data,
             offset: 0,
             static_params: StaticParams {
@@ -454,6 +656,64 @@ impl AvfVideo {
                 chordings_s: 0.0,
                 clicks_s: 0.0,
                 ces_s: 0.0,
+                ioe: 0.0,
+                thrp: 0.0,
+                corr: 0.0,
+            },
+        }
+    }
+    #[cfg(feature = "js")]
+    pub fn new(video_data: Vec<u8>) -> AvfVideo {
+        // video_data = video_data.into_vec();
+        AvfVideo {
+            file_name: "".to_string(),
+            width: 0,
+            height: 0,
+            mine_num: 0,
+            marks: false,
+            level: 0,
+            board: vec![],
+            events: vec![],
+            current_event_id: 0,
+            player: "".to_string(),
+            start_time: "".to_string(),
+            end_time: "".to_string(),
+            video_data: video_data,
+            offset: 0,
+            static_params: StaticParams {
+                bbbv: 0,
+                openings: 0,
+                islands: 0,
+                hizi: 0,
+                cell0: 0,
+                cell1: 0,
+                cell2: 0,
+                cell3: 0,
+                cell4: 0,
+                cell5: 0,
+                cell6: 0,
+                cell7: 0,
+                cell8: 0,
+            },
+            dynamic_params: DynamicParams {
+                r_time: 0.0,
+                bbbv_s: 0.0,
+                stnb: 0.0,
+                rqp: 0.0,
+                lefts: 0,
+                rights: 0,
+                chordings: 0,
+                flags: 0,
+                clicks: 0,
+                ces: 0,
+                lefts_s: 0.0,
+                rights_s: 0.0,
+                chordings_s: 0.0,
+                clicks_s: 0.0,
+                ces_s: 0.0,
+                ioe: 0.0,
+                thrp: 0.0,
+                corr: 0.0,
             },
         }
     }
@@ -522,13 +782,38 @@ impl AvfVideo {
                 break;
             }
         }
-        let mut buffer: [char; 2] = ['\0', '\0'];
         loop {
-            buffer[0] = buffer[1];
-            buffer[1] = self.get_char()?;
+            let v = self.get_char()?;
+            match v {
+                '|' => break,
+                _ => self.start_time.push(v),
+            }
+        }
+        // println!("666");
+        // loop {
+        //     let v = self.get_char()?;
+        //     print!("{:?}", v as char);
+        // }
+        loop {
+            let v = self.get_char()?;
+            match v {
+                '|' => break,
+                _ => self.end_time.push(v),
+            }
+        }
+        let v = self.get_char()?;
+        let mut buffer: [char; 2];
+        match v {
+            '|' => buffer = ['\0', '|'],
+            'B' => buffer = ['|', 'B'],
+            _ => buffer = ['\0', '\0'],
+        }
+        loop {
             if buffer[0] == '|' && buffer[1] == 'B' {
                 break;
             }
+            buffer[0] = buffer[1];
+            buffer[1] = self.get_char()?;
         }
         let mut s: String = "".to_string();
         loop {
@@ -554,6 +839,7 @@ impl AvfVideo {
             Ok(v) => v - 1.0,
             Err(_) => return Err(ErrReadVideoReason::InvalidParams),
         };
+        // println!("777");
         let mut buffer = [0u8; 8];
         while buffer[2] != 1 || buffer[1] > 1 {
             buffer[0] = buffer[1];
@@ -564,6 +850,9 @@ impl AvfVideo {
             buffer[i] = self.get_unsized_int()?;
         }
         loop {
+            // if buffer[0] != 1 {
+            // println!("{:?}, {:?}", ((buffer[6] as u16) << 8 | buffer[2] as u16) as f64 - 1.0
+            // + (buffer[4] as f64) / 100.0, buffer[0]);}
             self.events.push(VideoEvent {
                 time: ((buffer[6] as u16) << 8 | buffer[2] as u16) as f64 - 1.0
                     + (buffer[4] as f64) / 100.0,
@@ -575,7 +864,7 @@ impl AvfVideo {
                     17 => "rr".to_string(),
                     33 => "mc".to_string(),
                     65 => "mr".to_string(),
-                    145 => "rc".to_string(),
+                    145 => "rr".to_string(),
                     193 => "mr".to_string(),
                     11 => "sc".to_string(),
                     21 => "lr".to_string(),
@@ -588,6 +877,7 @@ impl AvfVideo {
                 useful_level: 0,
                 posteriori_game_board: GameBoard::new(self.mine_num),
                 comments: "".to_string(),
+                mouse_state: MouseState::Undefined,
             });
             for i in 0..8 {
                 buffer[i] = self.get_unsized_int()?;
@@ -596,7 +886,20 @@ impl AvfVideo {
                 break;
             }
         }
-        // 后面还有皮肤和标识符，暂时就不解析了
+        // 标识符
+        while self.get_char()? != 'S' {}
+        while self.get_char()? != 'k' {}
+        while self.get_char()? != 'i' {}
+        while self.get_char()? != 'n' {}
+        while self.get_char()? != ':' {}
+        while self.get_char()? != '\r' {}
+        loop {
+            let v = self.get_char()?;
+            match v {
+                '\r' => break,
+                _ => self.player.push(v),
+            }
+        }
         // for i in 0..1000 {
         //     for j in 0..8 {
         //         print!("{:?},", self.get_char().unwrap() as u8);
@@ -610,6 +913,7 @@ impl AvfVideo {
         let mut b = MinesweeperBoard::new(self.board.clone());
         for ide in 0..self.events.len() {
             if self.events[ide].mouse != "mv" {
+                // println!("{:?}, {:?}", self.events[ide].time, self.events[ide].mouse);
                 self.events[ide].useful_level = b
                     .step(
                         &self.events[ide].mouse,
@@ -623,20 +927,46 @@ impl AvfVideo {
                     .posteriori_game_board
                     .set_game_board(&b.game_board);
             }
+            self.events[ide].mouse_state = b.mouse_state.clone();
         }
         self.dynamic_params.lefts = b.left;
+        self.dynamic_params.lefts_s = b.left as f64 / self.dynamic_params.r_time;
         self.dynamic_params.rights = b.right;
+        self.dynamic_params.rights_s = b.right as f64 / self.dynamic_params.r_time;
         self.dynamic_params.ces = b.ces;
+        self.dynamic_params.ces_s = b.ces as f64 / self.dynamic_params.r_time;
         self.dynamic_params.chordings = b.chording;
+        self.dynamic_params.clicks = b.left + b.right + b.chording;
+        self.dynamic_params.clicks_s =
+            self.dynamic_params.clicks as f64 / self.dynamic_params.r_time;
         self.dynamic_params.flags = b.flag;
+        self.dynamic_params.bbbv_s = self.static_params.bbbv as f64 / self.dynamic_params.r_time;
+        self.dynamic_params.rqp = self.dynamic_params.r_time * self.dynamic_params.r_time
+            / self.static_params.bbbv as f64;
+        if self.height == 8 && self.width == 8 && self.mine_num == 10 {
+            self.dynamic_params.stnb = 47.22
+                / (self.dynamic_params.r_time.powf(1.7) / self.static_params.bbbv as f64)
+                * (b.solved3BV as f64 / self.static_params.bbbv as f64);
+        } else if self.height == 16 && self.width == 16 && self.mine_num == 40 {
+            self.dynamic_params.stnb = 153.73
+                / (self.dynamic_params.r_time.powf(1.7) / self.static_params.bbbv as f64)
+                * (b.solved3BV as f64 / self.static_params.bbbv as f64);
+        } else if self.height == 16 && self.width == 30 && self.mine_num == 99 {
+            self.dynamic_params.stnb = 435.001
+                / (self.dynamic_params.r_time.powf(1.7) / self.static_params.bbbv as f64)
+                * (b.solved3BV as f64 / self.static_params.bbbv as f64);
+        } // 凡自定义的stnb都等于0
+        self.dynamic_params.ioe = b.solved3BV as f64 / self.dynamic_params.clicks as f64;
+        self.dynamic_params.corr = b.ces as f64 / self.dynamic_params.clicks as f64;
+        self.dynamic_params.thrp = b.solved3BV as f64 / b.ces as f64;
     }
     pub fn analyse_for_features(&mut self, controller: Vec<&str>) {
         // 事件分析，返回一个向量，格式是event索引、字符串event的类型
         // error: 高风险的猜雷（猜对概率0.05）
-        // good: 跳判
+        // feature: 跳判
         // error: 判雷失败
-        // good: 双线操作
-        // good: 破空（成功率0.98）
+        // feature: 双线操作
+        // feature: 破空（成功率0.98）
         // error: 鼠标大幅绕路(500%)
         // warning：鼠标绕路(200%)
         // warning: 可以判雷时选择猜雷
@@ -656,18 +986,29 @@ impl AvfVideo {
         }
     }
     pub fn print_event(&self) {
+        let mut num = 0;
         for e in &self.events {
-            if e.mouse != "mv" {
-                println!("time = {:?}, mouse = {:?}, x = {:?}, y = {:?}", e.time, e.mouse, e.x, e.y);
-                e.posteriori_game_board
-                    .game_board
-                    .iter()
-                    .for_each(|v| println!("{:?}", v));
-                e.posteriori_game_board
-                    .poss
-                    .iter()
-                    .for_each(|v| println!("{:?}", v));
+            if num < 200 {
+                println!(
+                    "time = {:?}, mouse = {:?}, x = {:?}, y = {:?}, level = {:?}",
+                    e.time, e.mouse, e.x, e.y, e.useful_level
+                );
             }
+            num += 1;
+            // if e.mouse != "mv" {
+            //     println!(
+            //         "time = {:?}, mouse = {:?}, x = {:?}, y = {:?}",
+            //         e.time, e.mouse, e.x, e.y
+            //     );
+            //     e.posteriori_game_board
+            //         .game_board
+            //         .iter()
+            //         .for_each(|v| println!("{:?}", v));
+            //     e.posteriori_game_board
+            //         .poss
+            //         .iter()
+            //         .for_each(|v| println!("{:?}", v));
+            // }
         }
     }
     pub fn print_comments(&self) {
@@ -679,11 +1020,84 @@ impl AvfVideo {
     }
 }
 
+// 再实现一些get、set方法
+impl AvfVideo {
+    /// 获取当前录像时刻的游戏局面
+    pub fn get_game_board(&self) -> Vec<Vec<i32>> {
+        let mut id = self.current_event_id;
+        loop {
+            if self.events[id].posteriori_game_board.game_board.is_empty() {
+                id -= 1;
+            } else {
+                return self.events[id].posteriori_game_board.game_board.clone();
+            }
+        }
+    }
+    /// 获取当前录像时刻的局面概率
+    pub fn get_game_board_poss(&mut self) -> Vec<Vec<f64>> {
+        let mut id = self.current_event_id;
+        loop {
+            if self.events[id].useful_level < 2 {
+                id -= 1;
+                if id <= 0 {
+                    let p = self.mine_num as f64 / (self.height * self.width) as f64;
+                    return vec![vec![p; self.height]; self.width];
+                }
+            } else {
+                return self.events[id].posteriori_game_board.get_poss().clone();
+            }
+        }
+    }
+    /// 按时间设置current_event_id；超出两端范围取两端。
+    pub fn set_current_event_time(&mut self, time: f64) {
+        if time > self.events[self.current_event_id].time {
+            loop {
+                self.current_event_id += 1;
+                if self.current_event_id >= self.events.len() {
+                    self.current_event_id -= 1;
+                    return;
+                }
+                if self.events[self.current_event_id].time < time {
+                    continue;
+                } else {
+                    if self.events[self.current_event_id].time - time
+                        >= time - self.events[self.current_event_id - 1].time
+                    {
+                        self.current_event_id -= 1;
+                        return;
+                    } else {
+                        return;
+                    }
+                }
+            }
+        } else {
+            loop {
+                if self.current_event_id == 0 {
+                    return;
+                }
+                self.current_event_id -= 1;
+                if self.events[self.current_event_id].time > time {
+                    continue;
+                } else {
+                    if time - self.events[self.current_event_id].time
+                        >= self.events[self.current_event_id + 1].time - time
+                    {
+                        self.current_event_id += 1;
+                        return;
+                    } else {
+                        return;
+                    }
+                }
+            }
+        }
+    }
+}
+
 /// 静态游戏局面的包装类。  
 /// 所有计算过的属性都会保存在这里。  
 #[derive(Clone)]
 pub struct GameBoard {
-    game_board: Vec<Vec<i32>>,
+    pub game_board: Vec<Vec<i32>>,
     game_board_marked: Vec<Vec<i32>>,
     mine_num: usize,
     poss: Vec<Vec<f64>>,
